BuffDrop = {}
local core = BuffDrop

local frame = CreateFrame("Frame")
frame:RegisterEvent("PLAYER_ENTERING_WORLD")
frame:SetScript("OnEvent", function(this, event, ...)
    core[event](...)
end)

--Respond to events:
function core.PLAYER_ENTERING_WORLD()
	local function setup_db(db, defaults)
		for k,v in pairs(defaults) do
			if not db[k] then
				db[k] = v
			end
		end
		return db
	end
	if not BuffDropDB then BuffDropDB = {} end
	if not BuffDropPCDB then BuffDropPCDB = {} end
	core.db = setup_db(BuffDropDB, {
		icon = true,
		raid = true,
	})
	core.chardb = setup_db(BuffDropPCDB, {
		watch = {},
	})
	frame:UnregisterEvent("PLAYER_ENTERING_WORLD")
	
	frame:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_SELF")
end

local fade_pattern = AURAREMOVEDSELF:gsub("%%s", "(.-)") -- = "%s fades from you."
function core.CHAT_MSG_SPELL_AURA_GONE_SELF(msg)
	local buff = msg:match(fade_pattern)
	if buff and core.chardb.watch[buff] then
		core.AnnounceBuff(buff)
	end
end

local function get_buff_icon(buff)
	for i=1, BUFF_MAX_DISPLAY do
		if GetPlayerBuff(i) == 0 then break end
		if GetPlayerBuffName(i) == buff then
			return GetPlayerBuffTexture(i)
		end
	end
end
local color_red = {r=1, g=0, b=0}
function core.AnnounceBuff(buff)
	if core.db.raid then
		RaidNotice_AddMessage(RaidWarningFrame, buff, color_red)
	end
	if core.db.icon then
		local icon = get_buff_icon(buff)
		if icon then
			core.BigIcon(icon)
		end
	end
end

local bigicon
function core.BigIcon(icon)
	if not bigicon then
		frame:SetFrameStrata("HIGH")
		frame:SetPoint("CENTER", WorldFrame)
		local width = WorldFrame:GetHeight() / 3
		frame:SetWidth(width)
		frame:SetHeight(width)
		
		bigicon = frame:CreateTexture()
		bigicon:SetAllPoints(frame)
		bigicon:SetBlendMode("BLEND")
		bigicon:SetVertexColor(1, 1, 1, 0.5)
		bigicon:SetTexCoord(0.09, 0.91, 0.09, 0.91)
		
		local t = 0
		frame:SetScript("OnUpdate", function()
			t = t + arg1
			if t > 2 then
				bigicon:SetTexture(1, 0, 0)
				frame:Hide()
				t = 0
			end
		end)
		frame:Hide()
	end
	bigicon:SetTexture(icon)
	frame:Show()
end

local function announce(message, r, g, b)
	DEFAULT_CHAT_FRAME:AddMessage(message, r, g, b)
end
local menu_options = {
	icon = function(arg)
		if not arg then return " - big icon (" .. (core.db.icon and "true" or "false") .. ")" end
		core.db.icon = not core.db.icon
		announce("Huge icon "..(core.db.icon and "enabled" or "disabled"), 1, 1, 1)
	end,
	raid = function(arg)
		if not arg then return " - raid warning (" .. (core.db.raid and "true" or "false") .. ")" end
		core.db.raid = not core.db.raid
		announce("Raid warning "..(core.db.raid and "enabled" or "disabled"), 1, 1, 1)
	end,
	watch = function(arg)
		if not arg then return " - buffs to watch" end
		if arg == "" or arg:match("^%s+$") then
			announce("Watching:", 1, 1, 1)
			for buff in pairs(core.chardb.watch) do
				announce(" "..buff, 1, 1, 1)
			end
			return
		end
		if core.chardb.watch[arg] then
			core.chardb.watch[arg] = nil
			announce("No longer watching "..arg, 1, 1, 1)
		else
			core.chardb.watch[arg] = true
			announce("Now watching "..arg, 1, 1, 1)
		end
	end,
}
SLASH_BUFFDROP1 = "/buffdrop"
SLASH_BUFFDROP2 = "/bd"
SlashCmdList["BUFFDROP"] = function(arg)
	local command, arguments = string.match(arg, "^(%a+) ?(.*)")
	if menu_options[command] then
		return menu_options[command](arguments)
	end
	announce("BuffDrop:", 1, 1, 1)
	for k,v in pairs(menu_options) do
		announce(" -"..k..v(), 1, 1, 1)
	end
end
